@@ -23,9 +23,9 @@
 #include "db_storage.h"
 
 /* check_mandirs.c */
-extern void test_manfile (const char *file, const char *path);
+extern void test_manfile (MYDBM_FILE dbf, const char *file, const char *path);
 extern int create_db (const char *manpath, const char *catpath);
 extern int update_db (const char *manpath, const char *catpath);
-extern void purge_pointers (const char *name);
+extern void purge_pointers (MYDBM_FILE dbf, const char *name);
 extern int purge_missing (const char *manpath, const char *catpath,
 			  int will_run_mandb);
