@@ -170,8 +170,8 @@ static struct dirent_hashent *update_directory_cache (const char *path)
 		if (cache->names_len >= cache->names_max) {
 			cache->names_max *= 2;
 			cache->names =
-				xrealloc (cache->names,
-					  sizeof (char *) * cache->names_max);
+				xnrealloc (cache->names, cache->names_max,
+					   sizeof (char *));
 		}
 		cache->names[cache->names_len++] = xstrdup (entry->d_name);
 	}
@@ -216,7 +216,7 @@ static void match_in_directory (const char *path, const char *pattern, int opts,
 	size_t my_allocated = 0;
 	int flags;
 	regex_t preg;
-	struct pattern_bsearch pattern_start;
+	struct pattern_bsearch pattern_start = { NULL, -1 };
 	char **bsearched;
 	size_t i;
 
